#!/usr/bin/env python
# encoding: utf-8
#
# ** header v3.0
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2018 Research Group Biomedical Physics,
# Max-Planck-Institute for Dynamics and Self-Organization Göttingen
# Copyright (C) 2019 Henrik tom Wörden
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# ** end header
""" Defines how something that shall be inserted into CaosDB is treated.

CaosDB can automatically be filled with Records based on some file structure.
The Crawler will iterate over the files and test for each file whether a CFood
exists that matches the file path. If one does, it is instanciated to treat the
match. This occurs in basically three steps:
1. create a list of identifiables, i.e. unique representation of CaosDB Records
(such as an experiment belonging to a project and a date/time)
2. the identifiables are either found in CaosDB or they are created.
3. the identifiables are update based on the date in the file structure
"""

import argparse
import re
from argparse import RawTextHelpFormatter
from copy import deepcopy
from datetime import datetime

import caosdb as db

from caosadvancedtools.cache import Cache

ENTITIES = {}


def get_entity(name):
    """ Returns the entity with a given name, preferably from a local cache.

    If the local cache does not contain the entity, retrieve it from CaosDB.
    """

    if name not in ENTITIES:
        ent = db.Entity(name=name)
        ent.retrieve()
        ENTITIES[name] = ent

    return ENTITIES[name]


class AbstractCFood(object):
    # contains the compiled regular expression after the first execution of the
    # function match()
    _pattern = None

    def __init__(self, match, access=lambda x: x):
        """ Abstract base class for Crawler food (CFood).

        Parameters
        ----------
        match : match object of a regular expression match
                the result from matching a path against the pattern of this
                class

        access : callable, optional
                 A function that takes a CaosDB path and returns a local path
        """
        self.access = access
        self.crawled_file = match.string
        self.match = match

    @staticmethod
    def get_re():
        """ Returns the regular expression used to identify files that shall be
        processed

        This function shall be implemented by subclasses.
        """
        raise NotImplementedError()

    @classmethod
    def match(cls, string):
        """ Matches the regular expression of this class against file names

        Parameters
        ----------
        string : str
                 The path of the file that shall be matched.
        """

        if cls._pattern is None:
            cls._pattern = re.compile(cls.get_re())

        return cls._pattern.match(string)

    def create_identifiables(self):
        """
        must return a Container with the identifiables
        """
        raise NotImplementedError()

    def update_identifiables(self):
        raise NotImplementedError()

    @staticmethod
    # move to api?
    def set_parents(entity, names):
        entity.parents.clear()

        for n in names:
            entity.add_parent(get_entity(n))

    @staticmethod
    # move to api?
    def remove_property(entity, prop):
        # TODO only do something when it is necessary?

        if isinstance(prop, db.Entity):
            name = prop.name
        else:
            name = prop

        while entity.get_property(name) is not None:
            entity.remove_property(name)

    @staticmethod
    # move to api?
    def set_property(entity, prop, value, datatype=None):
        AbstractCFood.remove_property(entity, prop)

        if datatype is not None:
            entity.add_property(prop, value, datatype=datatype)
        else:
            entity.add_property(prop, value)
